// ERPNext - web based ERP (http://erpnext.com)
// Copyright (C) 2012 Web Notes Technologies Pvt Ltd
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

// Module CRM
cur_frm.cscript.tname = "Quotation Item";
cur_frm.cscript.fname = "quotation_details";
cur_frm.cscript.other_fname = "other_charges";
cur_frm.cscript.sales_team_fname = "sales_team";

// =====================================================================================
wn.require('app/accounts/doctype/sales_taxes_and_charges_master/sales_taxes_and_charges_master.js');
wn.require('app/utilities/doctype/sms_control/sms_control.js');
wn.require('app/selling/doctype/sales_common/sales_common.js');

erpnext.selling.QuotationController = erpnext.selling.SellingController.extend({
	onload: function(doc, dt, dn) {
		this._super(doc, dt, dn);
		if(doc.customer && !doc.quotation_to)
			doc.quotation_to = "Customer";
		else if(doc.lead && !doc.quotation_to)
			doc.quotation_to = "Lead";
	
	},
	refresh: function(doc, dt, dn) {
		this._super(doc, dt, dn);

		cur_frm.dashboard.reset(doc);
		if(!doc.__islocal) {
			if(doc.status=="Converted" || doc.status=="Order Confirmed") {
				cur_frm.dashboard.set_headline_alert(wn._(doc.status), "alert-success", "icon-ok-sign");
			} else if(doc.status==="Order Lost") {
				cur_frm.dashboard.set_headline_alert(wn._(doc.status), "alert-danger", "icon-exclamation-sign");
			}
		}
		
		if(doc.docstatus == 1 && doc.status!=='Order Lost') {
			cur_frm.add_custom_button('Make Sales Order', cur_frm.cscript['Make Sales Order']);
			if(doc.status!=="Order Confirmed") {
				cur_frm.add_custom_button('Set as Lost', cur_frm.cscript['Declare Order Lost']);
			}
			cur_frm.add_custom_button('Send SMS', cur_frm.cscript.send_sms);
		}
		
		if (this.frm.doc.docstatus===0) {
			cur_frm.add_custom_button(wn._('From Opportunity'), 
				function() {
					wn.model.map_current_doc({
						method: "selling.doctype.opportunity.opportunity.make_quotation",
						source_doctype: "Opportunity",
						get_query_filters: {
							docstatus: 1,
							status: "Submitted",
							enquiry_type: cur_frm.doc.order_type,
							customer: cur_frm.doc.customer || undefined,
							lead: cur_frm.doc.lead || undefined,
							company: cur_frm.doc.company
						}
					})
				});
		}
		

		if (!doc.__islocal) {
			cur_frm.communication_view = new wn.views.CommunicationList({
				list: wn.model.get("Communication", {"quotation": doc.name}),
				parent: cur_frm.fields_dict.communication_html.wrapper,
				doc: doc,
				recipients: doc.contact_email
			});		
		}
		
		this.quotation_to();
	},
	
	quotation_to: function() {
		this.frm.toggle_reqd("lead", this.frm.doc.quotation_to == "Lead");
		this.frm.toggle_reqd("customer", this.frm.doc.quotation_to == "Customer");
	},

	tc_name: function() {
		this.get_terms();
	},
	
	validate_company_and_party: function(party_field) {
		if(this.frm.doc.quotation_to == "Lead") {
			return true;
		} else if(!this.frm.doc.quotation_to) {
			msgprint(wn._("Please select a value for" + " " + wn.meta.get_label(this.frm.doc.doctype,
				"quotation_to", this.frm.doc.name)));
			return false;
		} else {
			return this._super(party_field);
		}
	},
});

// for backward compatibility: combine new and previous states
$.extend(cur_frm.cscript, new erpnext.selling.QuotationController({frm: cur_frm}));

cur_frm.cscript.customer_address = cur_frm.cscript.contact_person = function(doc,dt,dn) {
	if(doc.customer) get_server_fields('get_customer_address', JSON.stringify({
		customer: doc.customer, 
		address: doc.customer_address, 
		contact: doc.contact_person
	}),'', doc, dt, dn, 1);
}

cur_frm.fields_dict.lead.get_query = function(doc,cdt,cdn) {
	return{	query:"controllers.queries.lead_query" } }

cur_frm.cscript.lead = function(doc, cdt, cdn) {
	if(doc.lead) {
		cur_frm.call({
			doc: cur_frm.doc,
			method: "set_lead_defaults",
			callback: function(r) {
				if(!r.exc) {
					cur_frm.refresh_fields();
				}
			}
		});
		unhide_field('territory');
	}
}


// Make Sales Order
// =====================================================================================
cur_frm.cscript['Make Sales Order'] = function() {
	wn.model.open_mapped_doc({
		method: "selling.doctype.quotation.quotation.make_sales_order",
		source_name: cur_frm.doc.name
	})
}

// declare order lost
//-------------------------
cur_frm.cscript['Declare Order Lost'] = function(){
	var dialog = new wn.ui.Dialog({
		title: "Set as Lost",
		fields: [
			{"fieldtype": "Text", "label": "Reason for losing", "fieldname": "reason",
				"reqd": 1 },
			{"fieldtype": "Button", "label": "Update", "fieldname": "update"},
		]
	});

	dialog.fields_dict.update.$input.click(function() {
		args = dialog.get_values();
		if(!args) return;
		cur_frm.call({
			method: "declare_order_lost",
			doc: cur_frm.doc,
			args: args.reason,
			callback: function(r) {
				if(r.exc) {
					msgprint("There were errors.");
					return;
				}
				dialog.hide();
				cur_frm.refresh();
			},
			btn: this
		})
	});
	dialog.show();
	
}

//================ Last Quoted Price and Last Sold Price suggestion ======================
cur_frm.fields_dict['quotation_details'].grid.get_field('item_code').get_query= function(doc, cdt, cdn) {
	var d = locals[cdt][cdn];
	var cond = (doc.order_type == 'Maintenance') ? " and item.is_service_item = 'Yes'" : " and item.is_sales_item = 'Yes'";
	if(doc.customer) {
		var export_rate_field = wn.meta.get_docfield(cdt, 'export_rate', cdn);
		var precision = (export_rate_field && export_rate_field.fieldtype) === 'Float' ? 6 : 2;
		return {
			query: "selling.doctype.quotation.quotation.quotation_details",
			filters:{
				cust: doc.customer,
				cond: cond,
				precision: precision
			}
		}
	} else {
		return {
			query: 'selling.doctype.quotation.quotation.quotation_details',
			filters:{
				cond: cond,
			}		
		}	
		// return repl("SELECT name, item_name, description FROM `tabItem` item WHERE item.%(key)s LIKE '%s' %(cond)s ORDER BY item.item_code DESC LIMIT 50", {cond:cond});
	}
}

cur_frm.cscript.on_submit = function(doc, cdt, cdn) {
	if(cint(wn.boot.notification_settings.quotation)) {
		cur_frm.email_doc(wn.boot.notification_settings.quotation_message);
	}
}